"""
This module is used to represent the control logic data in 2-dimensional form. 
The data values are represented as colors in the graph.
"""

import matplotlib.pyplot as plt
import matplotlib.animation as animation
import seaborn as sns
import matplotlib

class ChessImageGenerator:
    "Class to create the gif"

    @staticmethod
    def create_gif(gameData):
        "Prepare the rows and columns for the graph"
        matplotlib.use('agg')
        
        gameDataMapWhite = gameData["white"]
        gameDataMapBlack = gameData["black"]

        filename = gameData["filename"]

        gameMaxControlWhite = gameData["max_white_value"]
        gameMaxControlBlack = gameData["max_black_value"]

        xlabels = ["a", "b", "c", "d", "e", "f", "g", "h"]
        ylabels = ["8", "7", "6", "5", "4", "3", "2", "1"]

        fig, (ax1, ax2) = plt.subplots(1, 2)

        sns.heatmap(gameDataMapWhite[0], cmap="YlGn", vmax = gameMaxControlWhite, xticklabels=xlabels,yticklabels=ylabels, ax = ax1)
        sns.heatmap(gameDataMapBlack[0], cmap="YlGn", vmax = gameMaxControlBlack, xticklabels=xlabels,yticklabels=ylabels, ax = ax2)

        def animate(i):
            "Create a single animated gif containing the heatmaps for all the different plys in a single game"
            ax1.cla()
            ax2.cla()
            sns.heatmap(gameDataMapWhite[i], cmap="YlGn", vmax = gameMaxControlWhite, xticklabels=xlabels,yticklabels=ylabels, ax = ax1, cbar = None)
            sns.heatmap(gameDataMapBlack[i], cmap="YlGn", vmax = gameMaxControlBlack, xticklabels=xlabels,yticklabels=ylabels, ax = ax2, cbar = None)

        anim = animation.FuncAnimation(fig, animate, interval=1000, save_count = len(gameDataMapWhite))
        anim.save("resources/output/" + filename + ".gif", dpi=80, writer='imagemagick')



class MockAnimate:    
    passed_filenames = []
    passed_kwargs = []
   
    def save(self,filename,**kwargs):
        self.passed_filenames.append(filename)
        self.passed_kwargs.append(kwargs)


    
    



"""
This module will handle logic pertaining to calculation of control of white and black over each square in the board
""" 
import glob
from chess import pgn
from chess import parse_square
from chess import Board
from chess import Piece
from chess import WHITE
from chess import BLACK
from chess import SQUARE_NAMES
from chess import Move
from chess import KING

class ChessUtil:
    "Class to publish (num_white_control,num_black_control) tuple for each square for each ply"

    @staticmethod
    def generate_ply_info_list_for_game(game):
        "Returns a dict which contains the list of tasks to be run and the number of plies for the game."
        board = Board()
        ply_no = 0
        game_tasks = []
        
        for ply in game.mainline_moves():
            board.push(ply)
            for square in SQUARE_NAMES:
                new_board = board.copy()
                parsed_square = parse_square(square)
                ply_info = {"ply_no": ply_no, "square": parsed_square, "board": new_board}
                game_tasks.append(ply_info)
            ply_no = ply_no + 1

        return {"ply_info_list": game_tasks, "ply_count": ply_no}

    @staticmethod
    def find_control_for_square_for_color(ply_info, color):
        "Calculate the number of attackers for each square for a ply"
        board = ply_info["board"]
        parsed_square = ply_info["square"]
        attackers = board.attackers(color, parsed_square)
        if len(attackers) == 0:
            return 0
        
        power_of_square = 0
        newBoard = board.copy()
        while(len(attackers) != 0):
            attacker_list = list(attackers)
            attacking_square = attacker_list[0]

            if newBoard.piece_type_at(attacking_square) == KING and len(attackers) > 1:
                attacking_square = attacker_list[1]
            elif newBoard.piece_type_at(attacking_square) == KING and len(attackers) == 1:
                power_of_square = power_of_square + 1
                break
            newBoard.remove_piece_at(attacking_square)
            power_of_square = power_of_square + 1
            attackers = newBoard.attackers(color, parsed_square)
        return power_of_square

    @staticmethod
    def find_control_for_square(ply_info):
        power_of_square = {}
        power_of_square['square'] = ply_info['square']
        power_of_square["white"] = ChessUtil.find_control_for_square_for_color(ply_info, WHITE)
        power_of_square["black"] = ChessUtil.find_control_for_square_for_color(ply_info, BLACK)
        power_of_square['ply'] = ply_info['ply_no']
        return power_of_square

    @staticmethod
    def get_games_from_pgn_files():
        "Parse PGN files in the current directory and return a list of parsed game objects"
        game_list = []
        for file in glob.glob("resources/input/*.pgn"):
            file_handle = open(file)
            while True:
                game = pgn.read_game(file_handle)
                if game is None:
                    break
                game_list.append(game)
        return game_list



"""
This module tests the method that calculates the number of attackers for each square.
Input is a CSV file
"""
import os
import sys
import csv
import chess
import pytest
from chess import parse_square
sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
from chess_util import ChessUtil

def get_data():
    "Fetches data from a CSV file"
    ret_list = []
    csv_file = os.path.join(os.path.dirname(os.path.abspath(__file__)),'board.csv')
    with open(csv_file) as csvfile:
        boardreader = csv.reader(csvfile, delimiter=',')
        for row in boardreader:
            ret_list.append(row)
    return ret_list

@pytest.mark.parametrize("fen,square,color,expected", get_data())
def test_calculate_control_for_square_multiple_values(fen, square, color, expected):
    """
    Given a game and a color is specified, when a specific square is considered,
    then the number of attackers for that square should be the number of pieces
    of that color which can move to the specified square
    """
    board = chess.Board(fen)
    parsed_square = parse_square(square)
    ply_no = 0
    ply_info = {"ply_no": ply_no, "square": parsed_square, "board": board}

    square_power = ChessUtil.find_control_for_square_for_color(ply_info, color == 'W')
    assert str(square_power) == expected



"""
This module contains unit tests the method part of the Chess image generator class.
"""
import sys
import os
import unittest
from unittest.mock import patch, call
from mock_animate import MockAnimate
sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
from chess_image_generator import ChessImageGenerator


class TestChessImageGenerator(unittest.TestCase):
    "This class tests the method that creates the gif"

    @patch('chess_image_generator.animation.FuncAnimation')
    @patch('chess_image_generator.sns.heatmap')
    @patch('chess_image_generator.plt.subplots')
    @patch('matplotlib.use')
    def test_create_gif(self, mocked_graph, mocked_plot, mocked_sns_heatmap, mocked_func_animate):
        mocked_graph.return_value = None
        mocked_sns_heatmap.return_value = None
        animate_obj = MockAnimate()
        mocked_func_animate.return_value = animate_obj
        ax1 = 1
        ax2 = 2
        fig = 3
        mocked_plot.return_value = [fig, (ax1, ax2)]
        ply1 = [[1, 2, 3, 4, 5, 6, 7, 8], [1, 2, 3, 4, 5, 6, 7, 8],
                [1, 2, 3, 4, 5, 6, 7, 8], [1, 2, 3, 4, 5, 6, 7, 8],
                [1, 2, 3, 4, 5, 6, 7, 8], [1, 2, 3, 4, 5, 6, 7, 8],
                [1, 2, 3, 4, 5, 6, 7, 8], [1, 2, 3, 4, 5, 6, 7, 8]]

        xlabels = ["a", "b", "c", "d", "e", "f", "g", "h"]
        ylabels = ["8", "7", "6", "5", "4", "3", "2", "1"]

        mocked_game_data = {"white": [ply1, ply1, ply1], "black": [ply1, ply1, ply1],
                            "max_white_value": 3, "max_black_value": 4, "filename": "Game0"}
        ChessImageGenerator.create_gif(mocked_game_data)
        mocked_graph.assert_called_with('agg')
        mocked_plot.assert_called_with(1, 2)
        mocked_sns_heatmap.assert_has_calls([call(ply1, cmap="YlGn", vmax=3, xticklabels=xlabels,
                                                  yticklabels=ylabels, ax=1),
                                             call(ply1, cmap="YlGn", vmax=4, xticklabels=xlabels,
                                                  yticklabels=ylabels, ax=2)])


if __name__ == '__main__':
    unittest.main()



"""
This module is for parallelizing the calculation of power of each square (in a ply) using dask tasks and parallelizing 
multiple games in PGNs
"""

from dask.distributed import LocalCluster
from dask.distributed import Client
from dask.distributed import worker_client
from distributed import wait
import coiled
from .chess_util import ChessUtil
from .chess_image_generator import ChessImageGenerator

class ChessDaskCluster:
    client: Client

    @staticmethod
    def get_game_data(result_list):
        #Convert the passed list of control per square per ply for all plies in the game into an object that is used for gif creation
        
        gameDataMapWhite = []
        gameDataMapBlack = []
        gameMaxControlWhite = 0
        gameMaxControlBlack = 0
        plyMapWhite = [[0 for x in range(8)] for y in range(8)]
        plyMapBlack = [[0 for x in range(8)] for y in range(8)]
        columnIndex = 0
        row = []
        rowIndex = 0
        plyNo = 0
        for result in result_list:
            square_index = result["square"]
            row = square_index//8
            column = square_index%8
            if result["white"] > gameMaxControlWhite:
                gameMaxControlWhite = result["white"]
            if result["black"] > gameMaxControlBlack:
                gameMaxControlBlack = result["black"]
            plyMapWhite[7-row][column] = result["white"]
            plyMapBlack[7-row][column] = result["black"]
            rowIndex = rowIndex + 1
            if rowIndex == 8:
                rowIndex = 0
                columnIndex = columnIndex + 1
                if columnIndex == 8:
                    columnIndex = 0
                    gameDataMapWhite.append(plyMapWhite)
                    gameDataMapBlack.append(plyMapBlack)
                    plyNo = plyNo + 1
                    plyMapWhite = [[0 for x in range(8)] for y in range(8)]
                    plyMapBlack = [[0 for x in range(8)] for y in range(8)]
        return {"white": gameDataMapWhite, "black": gameDataMapBlack, "max_white_value": gameMaxControlWhite, "max_black_value": gameMaxControlBlack}

    @staticmethod
    def run_in_parallel(game, game_no):
        """
        For the given game, for every ply, generate tasks to be run in parallel in a dask cluster. One task is created per square
        to find the control of both the sides. A worker client is used here because this method itself is run inside a worker (called
        from analyse_games_in_cluster)
        """
        task_futures = []
        tasks_for_game = ChessUtil.generate_ply_info_list_for_game(game)
        with worker_client() as client:
            for game_task in tasks_for_game["ply_info_list"]:
                task_futures.append(
                    client.submit(ChessUtil.find_control_for_square, game_task))

        wait(task_futures)
        control_list_for_game = []
        for task_future in task_futures:
            control_list_for_game.append(task_future.result())
        game_data = ChessDaskCluster.get_game_data(control_list_for_game)
        game_data["filename"] = "Game" + str(game_no)
        ChessImageGenerator.create_gif(game_data)

    def __init__(self):
        coiled.create_software_environment(name="chess-env", pip="requirements.txt")
        # TODO: change to get the following from config, instead of hardcoding them
        cluster = coiled.Cluster(name="chess-cluster", n_workers=25, worker_cpu=4, worker_memory="16 GiB",
            scheduler_memory="16 GiB", software="chess-env")
        # TODO: have a boolean in config to either run in local or run in coiled
        # cluster = LocalCluster(n_workers=8, threads_per_worker=1)
        self.client = Client(cluster)

    def analyse_games_in_cluster(self, game_list):
        #Find control heatmap for all the games passed in parallel in a dask cluster
        game_no = 0
        game_futures = []
        for game in game_list:
            game_futures.append(self.client.submit(ChessDaskCluster.run_in_parallel, *(game, game_no)))
            game_no = game_no + 1
        wait(game_futures)


     



"""
This module contains unit tests for all the methods part of the Chess util class.
"""
import unittest
import glob
import sys
import os
import chess
import pytest
from mock import patch, call
from chess import WHITE
from chess import BLACK
from chess import parse_square
from chess import pgn
from chess import Board
from mock_game import MockGame
sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
from chess_util import ChessUtil

class TestChessUtil(unittest.TestCase):
    "This class tests the chess specific methods"

    @patch('glob.glob') 
    def test_reading_games_from_pgn_files(self, mockglob):
        """
        Given two pgn files, when one has 3 games and the other has 1 game,
        then 4 games should be read and Event names should match
        """
        file1 = os.path.join(os.path.dirname(os.path.abspath(__file__)),'input/Adams.pgn')
        file2 = os.path.join(os.path.dirname(os.path.abspath(__file__)),'input/Anand.pgn')
        mockglob.return_value = [file1, file2]
        games = ChessUtil.get_games_from_pgn_files()
        self.assertEqual(len(games), 4)
        game_names = ["Game1", "Game2", "Game3", "Game4"]
        i = 0
        for game in games:
            self.assertEqual(game.headers["Event"], game_names[i])
            i = i + 1

    def test_generating_tasks_for_a_game(self):
        """
        Given a game, when two plies are present in that game,
        then 128 tasks have to be created with valid squares and ply applied board
        """
        #MockGame contains two plies
        mocked_game = MockGame()
        game_ply_info = ChessUtil.generate_ply_info_list_for_game(mocked_game)
        self.assertEqual(len(game_ply_info["ply_info_list"]),128)
        self.assertEqual(game_ply_info["ply_count"],2)
        ply_no = 0
        fen = 'rnbqkbnr/pppppppp/8/8/8/1P6/P1PPPPPP/RNBQKBNR'

        count = 0
        for game_task in game_ply_info["ply_info_list"]:
            if count == 64:
                ply_no = ply_no + 1
                fen = 'rnbqkbnr/1ppppppp/p7/8/8/1P6/P1PPPPPP/RNBQKBNR'
            self.assertEqual(game_task["ply_no"],ply_no)
            self.assertEqual(game_task["board"].board_fen(),fen)
            count = count + 1

    def test_calculate_control_for_square_scneario(self):
        """
        Given a game and a color is specified, when a specific square is considered,
        then the number of attackers for that square should be the number of pieces of
        that color which can move to the specified square
        """
        board = chess.Board('rnbqkbnr/pppppppp/8/8/8/1P6/P1PPPPPP/RNBQKBNR w KQkq - 0 1')
        square = "f3"
        parsed_square = parse_square(square)
        ply_no = 0
        ply_info = {"ply_no": ply_no, "square": parsed_square, "board": board}

        square_power = ChessUtil.find_control_for_square_for_color(ply_info,WHITE)
        self.assertEqual(square_power,4)

    @patch('chess_util.ChessUtil.find_control_for_square_for_color')
    def test_control_for_square(self,mocked_control_info):
        """
        Given a game, when a specific square is considered,
        then the control should be calculated correctly for both White and Black"
        """
        mocked_control_info.side_effect = [2,5]
        board = chess.Board('8/2q2n2/8/8/pp2b3/8/1RP5/2B1K3 w - - 0 1')
        color = WHITE
        square = "d2"
        parsed_square = parse_square(square)
        ply_no = 0
        ply_info = {"ply_no": ply_no, "square": parsed_square, "color": color, "board": board}

        square_total_info = ChessUtil.find_control_for_square(ply_info)

        self.assertEqual(square_total_info["square"],11)
        self.assertEqual(square_total_info["white"],2)
        self.assertEqual(square_total_info["black"],5)
        self.assertEqual(square_total_info["ply"],0)

        mocked_control_info.assert_has_calls([call(ply_info, WHITE), call(ply_info, BLACK)])

if __name__ == '__main__':
    unittest.main()



"""
This module is used to test the control heatmap manually by sending mocked data.
"""
import os
import sys
import matplotlib.pyplot as plt
import matplotlib.animation as animation
import seaborn as sns
import matplotlib
sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))


class ChessHeatmap:
    "Class to create the gif"

    @staticmethod
    def create_gif(gameData):
        "Prepare the rows and columns for the graph"
        matplotlib.use('agg')
        
        gameDataMapWhite = gameData["white"]
        gameDataMapBlack = gameData["black"]

        filename = gameData["filename"]

        gameMaxControlWhite = gameData["max_white_value"]
        gameMaxControlBlack = gameData["max_black_value"]

        xlabels = ["a", "b", "c", "d", "e", "f", "g", "h"]
        ylabels = ["8", "7", "6", "5", "4", "3", "2", "1"]

        fig, (ax1, ax2) = plt.subplots(1, 2)

        sns.heatmap(gameDataMapWhite[0], cmap="YlGn", vmin = 0, vmax = gameMaxControlWhite, xticklabels=xlabels,yticklabels=ylabels, ax = ax1)
        sns.heatmap(gameDataMapBlack[0], cmap="YlGn", vmin = 0, vmax = gameMaxControlBlack, xticklabels=xlabels,yticklabels=ylabels, ax = ax2)

        def animate(i):
            "Create a single animated gif containing the heatmaps for all the different plys in a single game"
            ax1.cla()
            ax2.cla()
            sns.heatmap(gameDataMapWhite[i], cmap="YlGn", vmin = 0, vmax = gameMaxControlWhite, xticklabels=xlabels,yticklabels=ylabels, ax = ax1, cbar = None)
            sns.heatmap(gameDataMapBlack[i], cmap="YlGn", vmin = 0, vmax = gameMaxControlBlack, xticklabels=xlabels,yticklabels=ylabels, ax = ax2, cbar = None)

        anim = animation.FuncAnimation(fig, animate, interval=1000, save_count = len(gameDataMapWhite))
        anim.save("resources/output/" + filename + ".gif", dpi=80, writer='imagemagick')
    
    @staticmethod
    def generate_data():
        game_data = []
        for j in range(64):
            ply = []
            row = []
            for i in range(64):
                value_to_print = 0
                if j == i:
                    value_to_print = 1
                row.append(value_to_print)
                if (i + 1) % 8 == 0:
                    ply.append(row)
                    row = []
            game_data.append(ply)
            ply = []
        return game_data

if __name__ == '__main__':
    chess_heatmap = ChessHeatmap()
    ply0 = [0,0,0,0,0,0],[0,0,0,0,0,0],[0,0,0,0,0,0],[0,0,0,0,0,0],[0,0,0,0,0,0],[0,0,0,0,0,0],[0,0,0,0,0,0],[0,0,0,0,0,0]
    ply1 = [1,1,1,1,1,1],[1,1,1,1,1,1],[1,1,1,1,1,1],[1,1,1,1,1,1],[1,1,1,1,1,1],[1,1,1,1,1,1],[1,1,1,1,1,1],[1,1,1,1,1,1]
    ply2 = [2,2,2,2,2,2],[2,2,2,2,2,2],[2,2,2,2,2,2],[2,2,2,2,2,2],[2,2,2,2,2,2],[2,2,2,2,2,2],[2,2,2,2,2,2],[2,2,2,2,2,2]
    ply3 = [3,3,3,3,3,3],[3,3,3,3,3,3],[3,3,3,3,3,3],[3,3,3,3,3,3],[3,3,3,3,3,3],[3,3,3,3,3,3],[3,3,3,3,3,3],[3,3,3,3,3,3]
    ply4 = [4,4,4,4,4,4],[4,4,4,4,4,4],[4,4,4,4,4,4],[4,4,4,4,4,4],[4,4,4,4,4,4],[4,4,4,4,4,4],[4,4,4,4,4,4],[4,4,4,4,4,4]
    ply5 = [5,5,5,5,5,5],[5,5,5,5,5,5],[5,5,5,5,5,5],[5,5,5,5,5,5],[5,5,5,5,5,5],[5,5,5,5,5,5],[5,5,5,5,5,5],[5,5,5,5,5,5]
    
    plyData = ChessHeatmap.generate_data()
   
    #gameData = {"white":[plyData[0]], "black":[plyData[1]],"max_white_value":5,"max_black_value":5,"filename":"Game1"}

    gameData = {}
    plywhite = []
    plyblack = []
    for i in range(64):
        plywhite.append(plyData[i])
        plyblack.append(plyData[i])
    
    gameData["white"] = plywhite
    gameData["black"] = plyblack
    gameData["max_white_value"] = 5
    gameData["max_black_value"] = 5
    gameData["filename"] = "Game1"

    chess_heatmap.create_gif(gameData)


from mock_future import MockFuture

class MockClient:
    def __init__(self):
        self.count = 0
        self.passed_args = []
        self.result_list_to_be_sent_in_future = []

    def submit(self,*args):
        self.passed_args.append(args)
        future = MockFuture(self.result_list_to_be_sent_in_future[self.count])
        self.count = self.count + 1
        return future
        
        


"""
This module contains unit tests for all the methods part of the Chess dask cluster class.
"""
import sys
import os
import unittest
from unittest.mock import Mock
from unittest.mock import patch, call
from dask.distributed import LocalCluster
from mock_client import MockClient
from mock_future import MockFuture
sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
from chess_util import ChessUtil
from chess_dask_cluster import ChessDaskCluster

class TestDaskChessCluster(unittest.TestCase):
    "This class tests the methods that implement parallelism using dask tasks"

    def test_get_game_data(self):
        "This method verifies whether the data required to plot the map is generated correctly"

        mocked_ply_info = []
        ply_no = 0
        count = 0
        for square in range(128):
            mock_data = {}
            mock_data["square"] = count
            mock_data["white"] = 1
            mock_data["black"] = 2
            mock_data["ply"] = ply_no
            count = count + 1
            if square == 63:
                ply_no = 1
                count = 0
            mocked_ply_info.append(mock_data)

        mock_result_list = ChessDaskCluster.get_game_data(mocked_ply_info)

        self.assertEqual(mock_result_list["max_white_value"], 1)
        self.assertEqual(mock_result_list["max_black_value"], 2)

        for ply in mock_result_list["white"]:
            for row in ply:
                for square_control in row:
                    self.assertEqual(square_control, 1)

        for ply in mock_result_list["black"]:
            for row in ply:
                for square_control in row:
                    self.assertEqual(square_control, 2)

    @patch('chess_dask_cluster.ChessDaskCluster.__init__')
    @patch('chess_dask_cluster.wait')
    def test_analyse_games_in_cluster(self, mocked_wait, mocked_constructor):
        mocked_constructor.return_value = None
        mocked_wait.return_value = None
        mocked_game1 = {'game_name': "game1", "ply_count": 1}
        mocked_game2 = {'game_name': "game2", "ply_count": 2}
        game_list = [mocked_game1, mocked_game2]

        dask_cluster = ChessDaskCluster()
        mock_client = MockClient()
        dask_cluster.client = mock_client
        mock_client.result_list_to_be_sent_in_future = [1, 2]
        dask_cluster.analyse_games_in_cluster(game_list)

        passed_args = mock_client.passed_args
        self.assertEqual(len(passed_args), 2)
        for passed_arg in range(2):
            self.assertEqual(passed_args[passed_arg][0], dask_cluster.run_in_parallel)
            self.assertEqual(passed_args[passed_arg][1], game_list[passed_arg])
            self.assertEqual(passed_args[passed_arg][2], passed_arg)

    @patch('chess_dask_cluster.wait')
    @patch('chess_dask_cluster.worker_client')
    @patch('chess_dask_cluster.ChessDaskCluster.get_game_data')
    @patch('chess_util.ChessUtil.find_control_for_square')
    @patch('chess_util.ChessUtil.generate_ply_info_list_for_game')
    @patch('chess_image_generator.ChessImageGenerator.create_gif')
    def test_run_in_parallel(self, create_gif_mock, mocked_generated_info,
                             mocked_control, mocked_game_data, mocked_worker_client, mocked_wait):
        mocked_wait.return_value = None
        create_gif_mock.return_value = None
        mock_game_data = []
        mocked_game_no = 0
        mocked_game = {'game_name': "game1", "ply_count": 1}
        mocked_game_list = {'ply_info_list': ["game_task1", "game_task2"]}
        mocked_generated_info.return_value = mocked_game_list

        mock_client = MockClient()
        mocked_worker_client.return_value.__enter__.return_value = mock_client

        mock_client.result_list_to_be_sent_in_future = [1, 2]
        mocked_game_data.return_value = {"name": 'Chess_game'}
        ChessDaskCluster.run_in_parallel(mocked_game, mocked_game_no)
        passed_args = mock_client.passed_args
        self.assertEqual(len(passed_args), 2)
        for passed_arg in range(2):
            self.assertEqual(passed_args[passed_arg][0],
                             ChessUtil.find_control_for_square)
            self.assertEqual(passed_args[passed_arg][1],
                             mocked_game_list["ply_info_list"][passed_arg])
        mocked_game_data.assert_called_with([1, 2])
        create_gif_mock.assert_called_with(
            {"name": 'Chess_game', "filename": "Game0"})


if __name__ == "__main__":
    unittest.main()



class MockSnsHeatmap:
    passed_args = []
    passed_kwargs = []

    @staticmethod
    def heatmap(*args,**kwargs):
        passed_args.append(args)
        passed_kwargs.append(kwargs)
        print("calling method")
